package sun.dc;

import sun.dc.path.FastPathProducer;
import java.awt.geom.Path2D;
import sun.dc.pr.PRException;
import sun.java2d.pipe.AATileGenerator;
import sun.java2d.pipe.Region;
import java.awt.geom.PathIterator;
import sun.dc.path.PathConsumer;
import sun.dc.path.PathException;
import sun.awt.geom.PathConsumer2D;
import java.awt.BasicStroke;
import sun.dc.pr.PathDasher;
import sun.dc.pr.PathStroker;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import sun.dc.pr.Rasterizer;
import sun.java2d.pipe.RenderingEngine;

public class DuctusRenderingEngine extends RenderingEngine
{
    static final float PenUnits = 0.01f;
    static final int MinPenUnits = 100;
    static final int MinPenUnitsAA = 20;
    static final float MinPenSizeAA = 0.19999999f;
    static final float UPPER_BND = 1.7014117E38f;
    static final float LOWER_BND = -1.7014117E38f;
    private static final int[] RasterizerCaps;
    private static final int[] RasterizerCorners;
    private static Rasterizer theRasterizer;
    
    public DuctusRenderingEngine() {
    }
    
    static float[] getTransformMatrix(AffineTransform affineTransform) {
        float[] array = new float[4];
        double[] flatmatrix = new double[6];
        affineTransform.getMatrix(flatmatrix);
        for (int i = 0; i < 4; ++i) {
            array[i] = (float)flatmatrix[i];
        }
        return array;
    }
    
    @Override
    public Shape createStrokedShape(Shape shape, float penDiameter, int n, int n2, float n3, float[] array, float n4) {
        DuctusRenderingEngine.FillAdapter fillAdapter = new DuctusRenderingEngine.FillAdapter();
        PathStroker pathStroker = new PathStroker(fillAdapter);
        PathDasher pathDasher = null;
        try {
            pathStroker.setPenDiameter(penDiameter);
            pathStroker.setPenT4(null);
            pathStroker.setCaps(DuctusRenderingEngine.RasterizerCaps[n]);
            pathStroker.setCorners(DuctusRenderingEngine.RasterizerCorners[n2], n3);
            if (array != null) {
                pathDasher = new PathDasher(pathStroker);
                pathDasher.setDash(array, n4);
                pathDasher.setDashT4(null);
            }
            else {
                pathDasher = pathStroker;
            }
            this.feedConsumer(pathDasher, shape.getPathIterator(null));
        }
        finally {
            pathStroker.dispose();
            if (pathDasher != null) {
                pathDasher.dispose();
            }
        }
        return fillAdapter.getShape();
    }
    
    @Override
    public void strokeTo(Shape shape, AffineTransform affineTransform, BasicStroke basicStroke, boolean b, boolean b2, boolean b3, PathConsumer2D pathConsumer2D) {
        PathConsumer pathConsumer;
        PathStroker pathStroker = pathConsumer = new PathStroker(pathConsumer2D);
        float[] array = null;
        if (!b) {
            pathStroker.setPenDiameter(basicStroke.getLineWidth());
            if (affineTransform != null) {
                array = getTransformMatrix(affineTransform);
            }
            pathStroker.setPenT4(array);
            pathStroker.setPenFitting(0.01f, 100);
        }
        pathStroker.setCaps(DuctusRenderingEngine.RasterizerCaps[basicStroke.getEndCap()]);
        pathStroker.setCorners(DuctusRenderingEngine.RasterizerCorners[basicStroke.getLineJoin()], basicStroke.getMiterLimit());
        float[] dashArray = basicStroke.getDashArray();
        if (dashArray != null) {
            PathDasher o = new PathDasher(pathStroker);
            o.setDash(dashArray, basicStroke.getDashPhase());
            if (affineTransform != null && array == null) {
                array = getTransformMatrix(affineTransform);
            }
            o.setDashT4(array);
            pathConsumer = o;
        }
        try {
            feedConsumer(shape.getPathIterator(affineTransform), pathConsumer, b2, 0.25f);
            while (pathConsumer != null && pathConsumer != pathConsumer2D) {
                PathConsumer consumer = pathConsumer.getConsumer();
                pathConsumer.dispose();
                pathConsumer = consumer;
            }
        }
        catch (PathException cause) {
            throw new InternalError("Unable to Stroke shape (" + cause.getMessage() + ")", cause);
        }
        finally {
            while (pathConsumer != null && pathConsumer != pathConsumer2D) {
                PathConsumer consumer2 = pathConsumer.getConsumer();
                pathConsumer.dispose();
                pathConsumer = consumer2;
            }
        }
    }
    
    public static void feedConsumer(PathIterator pathIterator, PathConsumer pathConsumer, boolean b, float n) throws PathException {
        pathConsumer.beginPath();
        int n2 = 0;
        int n3 = 0;
        int n4 = 0;
        float n5 = 0.0f;
        float n6 = 0.0f;
        float[] array = new float[6];
        float n7 = 0.5f - n;
        float n8 = 0.0f;
        float n9 = 0.0f;
        while (!pathIterator.isDone()) {
            int currentSegment = pathIterator.currentSegment(array);
            if (n2 == 1) {
                n2 = 0;
                if (currentSegment != 0) {
                    pathConsumer.beginSubpath(n5, n6);
                    n4 = 1;
                }
            }
            if (b) {
                int n10;
                switch (currentSegment) {
                    case 3: {
                        n10 = 4;
                        break;
                    }
                    case 2: {
                        n10 = 2;
                        break;
                    }
                    case 0:
                    case 1: {
                        n10 = 0;
                        break;
                    }
                    default: {
                        n10 = -1;
                        break;
                    }
                }
                if (n10 >= 0) {
                    float n11 = array[n10];
                    float n12 = array[n10 + 1];
                    float n13 = (float)Math.floor((double)(n11 + n7)) + n;
                    float n14 = (float)Math.floor((double)(n12 + n7)) + n;
                    array[n10] = n13;
                    array[n10 + 1] = n14;
                    float n15 = n13 - n11;
                    float n16 = n14 - n12;
                    switch (currentSegment) {
                        case 3: {
                            int n17 = 0;
                            array[n17] += n8;
                            int n18 = 1;
                            array[n18] += n9;
                            int n19 = 2;
                            array[n19] += n15;
                            int n20 = 3;
                            array[n20] += n16;
                            break;
                        }
                        case 2: {
                            int n21 = 0;
                            array[n21] += (n15 + n8) / 2.0f;
                            int n22 = 1;
                            array[n22] += (n16 + n9) / 2.0f;
                            break;
                        }
                    }
                    n8 = n15;
                    n9 = n16;
                }
            }
            switch (currentSegment) {
                case 0: {
                    if (array[0] < 1.7014117E38f && array[0] > -1.7014117E38f && array[1] < 1.7014117E38f && array[1] > -1.7014117E38f) {
                        n5 = array[0];
                        n6 = array[1];
                        pathConsumer.beginSubpath(n5, n6);
                        n4 = 1;
                        n3 = 0;
                        break;
                    }
                    n3 = 1;
                    break;
                }
                case 1: {
                    if (array[0] < 1.7014117E38f && array[0] > -1.7014117E38f && array[1] < 1.7014117E38f && array[1] > -1.7014117E38f) {
                        if (n3 != 0) {
                            pathConsumer.beginSubpath(array[0], array[1]);
                            n4 = 1;
                            n3 = 0;
                            break;
                        }
                        pathConsumer.appendLine(array[0], array[1]);
                        break;
                    }
                    else {
                        break;
                    }
                }
                case 2: {
                    if (array[2] < 1.7014117E38f && array[2] > -1.7014117E38f && array[3] < 1.7014117E38f && array[3] > -1.7014117E38f) {
                        if (n3 != 0) {
                            pathConsumer.beginSubpath(array[2], array[3]);
                            n4 = 1;
                            n3 = 0;
                            break;
                        }
                        if (array[0] < 1.7014117E38f && array[0] > -1.7014117E38f && array[1] < 1.7014117E38f && array[1] > -1.7014117E38f) {
                            pathConsumer.appendQuadratic(array[0], array[1], array[2], array[3]);
                            break;
                        }
                        pathConsumer.appendLine(array[2], array[3]);
                        break;
                    }
                    else {
                        break;
                    }
                }
                case 3: {
                    if (array[4] < 1.7014117E38f && array[4] > -1.7014117E38f && array[5] < 1.7014117E38f && array[5] > -1.7014117E38f) {
                        if (n3 != 0) {
                            pathConsumer.beginSubpath(array[4], array[5]);
                            n4 = 1;
                            n3 = 0;
                            break;
                        }
                        if (array[0] < 1.7014117E38f && array[0] > -1.7014117E38f && array[1] < 1.7014117E38f && array[1] > -1.7014117E38f && array[2] < 1.7014117E38f && array[2] > -1.7014117E38f && array[3] < 1.7014117E38f && array[3] > -1.7014117E38f) {
                            pathConsumer.appendCubic(array[0], array[1], array[2], array[3], array[4], array[5]);
                            break;
                        }
                        pathConsumer.appendLine(array[4], array[5]);
                        break;
                    }
                    else {
                        break;
                    }
                }
                case 4: {
                    if (n4 != 0) {
                        pathConsumer.closedSubpath();
                        n4 = 0;
                        n2 = 1;
                        break;
                    }
                    break;
                }
            }
            pathIterator.next();
        }
        pathConsumer.endPath();
    }
    
    public static synchronized Rasterizer getRasterizer() {
        Rasterizer theRasterizer = DuctusRenderingEngine.theRasterizer;
        if (theRasterizer == null) {
            theRasterizer = new Rasterizer();
            return theRasterizer;
        }
        DuctusRenderingEngine.theRasterizer = null;
        return theRasterizer;
    }
    
    public static synchronized void dropRasterizer(Rasterizer theRasterizer) {
        theRasterizer.reset();
        DuctusRenderingEngine.theRasterizer = theRasterizer;
    }
    
    @Override
    public float getMinimumAAPenSize() {
        return 0.19999999f;
    }
    
    @Override
    public AATileGenerator getAATileGenerator(Shape shape, AffineTransform affineTransform, Region region, BasicStroke basicStroke, boolean b, boolean b2, int[] bbox) {
        Rasterizer rasterizer = getRasterizer();
        PathIterator pathIterator = shape.getPathIterator(affineTransform);
        if (basicStroke != null) {
            float[] array = null;
            rasterizer.setUsage(3);
            if (b) {
                rasterizer.setPenDiameter(0.19999999f);
            }
            else {
                rasterizer.setPenDiameter(basicStroke.getLineWidth());
                if (affineTransform != null) {
                    array = getTransformMatrix(affineTransform);
                    rasterizer.setPenT4(array);
                }
                rasterizer.setPenFitting(0.01f, 20);
            }
            rasterizer.setCaps(DuctusRenderingEngine.RasterizerCaps[basicStroke.getEndCap()]);
            rasterizer.setCorners(DuctusRenderingEngine.RasterizerCorners[basicStroke.getLineJoin()], basicStroke.getMiterLimit());
            float[] dashArray = basicStroke.getDashArray();
            if (dashArray != null) {
                rasterizer.setDash(dashArray, basicStroke.getDashPhase());
                if (affineTransform != null && array == null) {
                    array = getTransformMatrix(affineTransform);
                }
                rasterizer.setDashT4(array);
            }
        }
        else {
            rasterizer.setUsage((pathIterator.getWindingRule() == 0) ? 1 : 2);
        }
        rasterizer.beginPath();
        int n = 0;
        int n2 = 0;
        int n3 = 0;
        float n4 = 0.0f;
        float n5 = 0.0f;
        float[] array2 = new float[6];
        float n6 = 0.0f;
        float n7 = 0.0f;
        while (!pathIterator.isDone()) {
            int currentSegment = pathIterator.currentSegment(array2);
            if (n == 1) {
                n = 0;
                if (currentSegment != 0) {
                    rasterizer.beginSubpath(n4, n5);
                    n3 = 1;
                }
            }
            if (b2) {
                int n8;
                switch (currentSegment) {
                    case 3: {
                        n8 = 4;
                        break;
                    }
                    case 2: {
                        n8 = 2;
                        break;
                    }
                    case 0:
                    case 1: {
                        n8 = 0;
                        break;
                    }
                    default: {
                        n8 = -1;
                        break;
                    }
                }
                if (n8 >= 0) {
                    float n9 = array2[n8];
                    float n10 = array2[n8 + 1];
                    float n11 = (float)Math.floor((double)n9) + 0.5f;
                    float n12 = (float)Math.floor((double)n10) + 0.5f;
                    array2[n8] = n11;
                    array2[n8 + 1] = n12;
                    float n13 = n11 - n9;
                    float n14 = n12 - n10;
                    switch (currentSegment) {
                        case 3: {
                            int n15 = 0;
                            array2[n15] += n6;
                            int n16 = 1;
                            array2[n16] += n7;
                            int n17 = 2;
                            array2[n17] += n13;
                            int n18 = 3;
                            array2[n18] += n14;
                            break;
                        }
                        case 2: {
                            int n19 = 0;
                            array2[n19] += (n13 + n6) / 2.0f;
                            int n20 = 1;
                            array2[n20] += (n14 + n7) / 2.0f;
                            break;
                        }
                    }
                    n6 = n13;
                    n7 = n14;
                }
            }
            switch (currentSegment) {
                case 0: {
                    if (array2[0] < 1.7014117E38f && array2[0] > -1.7014117E38f && array2[1] < 1.7014117E38f && array2[1] > -1.7014117E38f) {
                        n4 = array2[0];
                        n5 = array2[1];
                        rasterizer.beginSubpath(n4, n5);
                        n3 = 1;
                        n2 = 0;
                        break;
                    }
                    n2 = 1;
                    break;
                }
                case 1: {
                    if (array2[0] < 1.7014117E38f && array2[0] > -1.7014117E38f && array2[1] < 1.7014117E38f && array2[1] > -1.7014117E38f) {
                        if (n2 != 0) {
                            rasterizer.beginSubpath(array2[0], array2[1]);
                            n3 = 1;
                            n2 = 0;
                            break;
                        }
                        rasterizer.appendLine(array2[0], array2[1]);
                        break;
                    }
                    else {
                        break;
                    }
                }
                case 2: {
                    if (array2[2] < 1.7014117E38f && array2[2] > -1.7014117E38f && array2[3] < 1.7014117E38f && array2[3] > -1.7014117E38f) {
                        if (n2 != 0) {
                            rasterizer.beginSubpath(array2[2], array2[3]);
                            n3 = 1;
                            n2 = 0;
                            break;
                        }
                        if (array2[0] < 1.7014117E38f && array2[0] > -1.7014117E38f && array2[1] < 1.7014117E38f && array2[1] > -1.7014117E38f) {
                            rasterizer.appendQuadratic(array2[0], array2[1], array2[2], array2[3]);
                            break;
                        }
                        rasterizer.appendLine(array2[2], array2[3]);
                        break;
                    }
                    else {
                        break;
                    }
                }
                case 3: {
                    if (array2[4] < 1.7014117E38f && array2[4] > -1.7014117E38f && array2[5] < 1.7014117E38f && array2[5] > -1.7014117E38f) {
                        if (n2 != 0) {
                            rasterizer.beginSubpath(array2[4], array2[5]);
                            n3 = 1;
                            n2 = 0;
                            break;
                        }
                        if (array2[0] < 1.7014117E38f && array2[0] > -1.7014117E38f && array2[1] < 1.7014117E38f && array2[1] > -1.7014117E38f && array2[2] < 1.7014117E38f && array2[2] > -1.7014117E38f && array2[3] < 1.7014117E38f && array2[3] > -1.7014117E38f) {
                            rasterizer.appendCubic(array2[0], array2[1], array2[2], array2[3], array2[4], array2[5]);
                            break;
                        }
                        rasterizer.appendLine(array2[4], array2[5]);
                        break;
                    }
                    else {
                        break;
                    }
                }
                case 4: {
                    if (n3 != 0) {
                        rasterizer.closedSubpath();
                        n3 = 0;
                        n = 1;
                        break;
                    }
                    break;
                }
            }
            pathIterator.next();
        }
        try {
            rasterizer.endPath();
            rasterizer.getAlphaBox(bbox);
            region.clipBoxToBounds(bbox);
            if (bbox[0] >= bbox[2] || bbox[1] >= bbox[3]) {
                dropRasterizer(rasterizer);
                return null;
            }
            rasterizer.setOutputArea((float)bbox[0], (float)bbox[1], bbox[2] - bbox[0], bbox[3] - bbox[1]);
        }
        catch (PRException obj) {
            System.err.println("DuctusRenderingEngine.getAATileGenerator: " + obj);
        }
        return rasterizer;
    }
    
    @Override
    public AATileGenerator getAATileGenerator(double n, double n2, double n3, double n4, double n5, double n6, double n7, double n8, Region region, int[] bbox) {
        int n9 = (n7 > 0.0 && n8 > 0.0) ? 1 : 0;
        double n13;
        double n12;
        double n11;
        double n10;
        if (n9 != 0) {
            n11 = n3 * n7;
            n10 = n4 * n7;
            n12 = n5 * n8;
            n13 = n6 * n8;
            n -= (n11 + n12) / 2.0;
            n2 -= (n10 + n13) / 2.0;
            n3 += n11;
            n4 += n10;
            n5 += n12;
            n6 += n13;
            if (n7 > 1.0 && n8 > 1.0) {
                n9 = 0;
            }
        }
        else {
            n10 = (n11 = (n12 = (n13 = 0.0)));
        }
        Rasterizer rasterizer = getRasterizer();
        rasterizer.setUsage(1);
        rasterizer.beginPath();
        rasterizer.beginSubpath((float)n, (float)n2);
        rasterizer.appendLine((float)(n + n3), (float)(n2 + n4));
        rasterizer.appendLine((float)(n + n3 + n5), (float)(n2 + n4 + n6));
        rasterizer.appendLine((float)(n + n5), (float)(n2 + n6));
        rasterizer.closedSubpath();
        if (n9 != 0) {
            n += n11 + n12;
            n2 += n10 + n13;
            n3 -= 2.0 * n11;
            n4 -= 2.0 * n10;
            n5 -= 2.0 * n12;
            n6 -= 2.0 * n13;
            rasterizer.beginSubpath((float)n, (float)n2);
            rasterizer.appendLine((float)(n + n3), (float)(n2 + n4));
            rasterizer.appendLine((float)(n + n3 + n5), (float)(n2 + n4 + n6));
            rasterizer.appendLine((float)(n + n5), (float)(n2 + n6));
            rasterizer.closedSubpath();
        }
        try {
            rasterizer.endPath();
            rasterizer.getAlphaBox(bbox);
            region.clipBoxToBounds(bbox);
            if (bbox[0] >= bbox[2] || bbox[1] >= bbox[3]) {
                dropRasterizer(rasterizer);
                return null;
            }
            rasterizer.setOutputArea((float)bbox[0], (float)bbox[1], bbox[2] - bbox[0], bbox[3] - bbox[1]);
        }
        catch (PRException obj) {
            System.err.println("DuctusRenderingEngine.getAATileGenerator: " + obj);
        }
        return rasterizer;
    }
    
    private void feedConsumer(PathConsumer pathConsumer, PathIterator pathIterator) {
        try {
            pathConsumer.beginPath();
            int n = 0;
            float n2 = 0.0f;
            float n3 = 0.0f;
            float[] array = new float[6];
            while (!pathIterator.isDone()) {
                int currentSegment = pathIterator.currentSegment(array);
                if (n == 1) {
                    n = 0;
                    if (currentSegment != 0) {
                        pathConsumer.beginSubpath(n2, n3);
                    }
                }
                switch (currentSegment) {
                    case 0: {
                        n2 = array[0];
                        n3 = array[1];
                        pathConsumer.beginSubpath(array[0], array[1]);
                        break;
                    }
                    case 1: {
                        pathConsumer.appendLine(array[0], array[1]);
                        break;
                    }
                    case 2: {
                        pathConsumer.appendQuadratic(array[0], array[1], array[2], array[3]);
                        break;
                    }
                    case 3: {
                        pathConsumer.appendCubic(array[0], array[1], array[2], array[3], array[4], array[5]);
                        break;
                    }
                    case 4: {
                        pathConsumer.closedSubpath();
                        n = 1;
                        break;
                    }
                }
                pathIterator.next();
            }
            pathConsumer.endPath();
        }
        catch (PathException cause) {
            throw new InternalError("Unable to Stroke shape (" + cause.getMessage() + ")", cause);
        }
    }
    
    static {
        RasterizerCaps = new int[] { 30, 10, 20 };
        RasterizerCorners = new int[] { 50, 10, 40 };
    }
    
    private class FillAdapter implements PathConsumer
    {
        boolean closed;
        Path2D.Float path;
        
        public FillAdapter() {
            this.path = new Path2D.Float(1);
        }
        
        public Shape getShape() {
            return this.path;
        }
        
        public void dispose() {
        }
        
        public PathConsumer getConsumer() {
            return null;
        }
        
        public void beginPath() {
        }
        
        public void beginSubpath(float x, float y) {
            if (this.closed) {
                this.path.closePath();
                this.closed = false;
            }
            this.path.moveTo(x, y);
        }
        
        public void appendLine(float x, float y) {
            this.path.lineTo(x, y);
        }
        
        public void appendQuadratic(float x1, float y1, float x2, float y2) {
            this.path.quadTo(x1, y1, x2, y2);
        }
        
        public void appendCubic(float x1, float y1, float x2, float y2, float x3, float y3) {
            this.path.curveTo(x1, y1, x2, y2, x3, y3);
        }
        
        public void closedSubpath() {
            this.closed = true;
        }
        
        public void endPath() {
            if (this.closed) {
                this.path.closePath();
                this.closed = false;
            }
        }
        
        public void useProxy(FastPathProducer fastPathProducer) throws PathException {
            fastPathProducer.sendTo(this);
        }
        
        public long getCPathConsumer() {
            return 0L;
        }
    }
}
